package eu.extech.objectdb;

import javax.jdo.*;
import javax.jdo.annotations.Index;
import javax.jdo.annotations.Unique;
import javax.persistence.Entity;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import java.util.*;

public class QueryByDateTest {

	public static void setUp() {
		// make cascading makeTransient/detach
		System.setProperty("objectdb.temp.detach-embeddable", "true");

		PersistenceManager pm = getPMF().getPersistenceManager();
		Extent<ClassWithDate> extent = pm.getExtent(ClassWithDate.class);
		pm.currentTransaction().begin();
		extent.iterator().forEachRemaining(object -> pm.deletePersistent(object));
		pm.currentTransaction().commit();
		pm.close();

		ClassWithDate instance = new ClassWithDate(TYPE, new Date());
		saveToDB(instance);
	}

	private static final String TYPE = "type1";

	public static void main(String[] args) {
		setUp();
		Date date = ClassWithDate.getCorrectDate(new Date());
		PersistenceManager pm = getPMF().getPersistenceManager();

		// these queries should find 1 item

		assertQueryResultSize(0, pm.newQuery(ClassWithDate.class, "this.date == datum"), date);
		assertQueryResultSize(0, pm.newQuery("SELECT FROM ClassWithDate WHERE date == datum PARAMETERS java.util.Date datum" ), date);
		assertQueryResultSize(1, pm.newQuery(ClassWithDate.class, "this.date.equals(datum)"), date);
		assertQueryResultSize(1, pm.newQuery(ClassWithDate.class, "this.type.equals(\""+ TYPE +"\") && this.date.equals(datum)"), date);

		// these queries should find 0 item
		assertQueryResultSize(0, pm.newQuery(ClassWithDate.class, "this.date < datum"), date);
		assertQueryResultSize(0, pm.newQuery(ClassWithDate.class, "this.date.before(datum)"), date);
		assertQueryResultSize(1, pm.newQuery(ClassWithDate.class, "this.date > datum"), date);
		assertQueryResultSize(1, pm.newQuery(ClassWithDate.class, "this.type.equals(\""+ TYPE + "\") && this.date > datum"), date);
		assertQueryResultSize(0, pm.newQuery(ClassWithDate.class, "this.date.after(datum)"), date);

		pm.close();
	}

	private static void assertQueryResultSize(int expectedResult, Query query, Date date) {
		query.declareParameters("java.util.Date datum");
		Collection<ClassWithDate> collection = (Collection) query.execute(date);
		if(expectedResult != collection.size()) {
			throw new IllegalStateException("Expected: " + expectedResult + " vs actual: " + collection.size());
		}

	}

	private static void saveToDB(ClassWithDate instance) {
		PersistenceManager pm = getPMF().getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(instance);
		pm.currentTransaction().commit();
		pm.close();
	}

	private static PersistenceManagerFactory pmf;
	private static PersistenceManagerFactory getPMF() {
		if(pmf == null) {
			pmf = JDOHelper.getPersistenceManagerFactory(getConnectionProperties(), JDOHelper.class.getClassLoader());
		}
		return pmf;
	}

	private static Properties getConnectionProperties() {
		Properties props = new Properties();
		props.put("javax.jdo.PersistenceManagerFactoryClass", "com.objectdb.jdo.PMF");
		props.put("javax.jdo.option.ConnectionURL", "test.odb");
		return props;
	}


	@Entity
	@Unique(members = {"type", "date"})
	public static class ClassWithDate {
		@Index
		private String type;

		@Index
		@Temporal(TemporalType.DATE)
		private Date date;

		public ClassWithDate(String type, Date date) {
			this.type = type;
			this.date = getCorrectDate(date);
		}

		public static Date getCorrectDate(Date date) {
			if (date == null) {
				return null;
			}
			GregorianCalendar calendar = new GregorianCalendar();
			calendar.setTime(date);
			calendar.set(Calendar.HOUR_OF_DAY, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);
			return calendar.getTime();
		}

	}

}
